@@ -39,6 +39,7 @@
 
 #include <Lum/Button.h>
 #include <Lum/Dialog.h>
+#include <Lum/Label.h>
 #include <Lum/String.h>
 #include <Lum/Table.h>
 #include <Lum/Text.h>
@@ -53,54 +54,144 @@ std::wstring             ossoVersion;
 
 static Lum::Def::AppInfo info;
 
-class PackageFormatProvider : public Lum::TableView::FormatProvider
+typedef Lum::Model::StdTable<Package*>     PackageModel;
+typedef Lum::Base::Reference<PackageModel> PackageModelRef;
+
+class PackageObject : public Lum::Object
 {
+private:
+  Package           *package;
+  Lum::OS::FontRef  font1;
+  Lum::OS::FontRef  font2;
+
 public:
-  Lum::TableView::Alignment GetAlignment(size_t column, size_t row) const
+  PackageObject()
+  : font1(Lum::OS::display->GetFont()),
+    font2(Lum::OS::display->GetFont(Lum::OS::Display::fontScaleFootnote))
+  {
+    // no code
+  }
+
+  void SetPackage(Package* package)
+  {
+    this->package=package;
+  }
+
+  void CalcSize()
+  {
+    minWidth=2*Lum::OS::display->GetSpaceHorizontal(Lum::OS::Display::spaceObjectBorder)+
+             10*font1->StringWidth(L"m",Lum::OS::Font::bold);
+    minHeight=Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
+              font1->height+
+              Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
+              font2->height+
+              Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder);
+
+    width=minWidth;
+    height=minHeight;
+
+    Object::CalcSize();
+  }
+
+  void Draw(int x, int y, size_t w, size_t h)
+  {
+    Object::Draw(x,y,w,h);
+
+    if (!OIntersect(x,y,w,h)) {
+      return;
+    }
+
+    /* --- */
+
+    Lum::OS::DrawInfo *draw=GetDrawInfo();
+
+    draw->PushForeground(parent->GetTextColor(draw));
+    draw->PushFont(font1,Lum::OS::Font::bold);
+    draw->DrawString(this->x,
+                     this->y+Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
+                     font1->ascent,
+                     package->GetName());
+
+    std::wstring version=package->GetVersion();
+
+    draw->DrawString(this->x+width-font1->StringWidth(version,Lum::OS::Font::bold)+1,
+                     this->y+Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
+                     font1->ascent,
+                     version);
+
+    draw->PopFont();
+
+    draw->PushFont(font2);
+    draw->DrawString(this->x,
+                     this->y+Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
+                     font1->height+
+                     Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
+                     font2->ascent,
+                     package->GetShortDescription());
+
+    std::wstring packageSize=Lum::Base::ByteSizeToWString(package->GetInstalledSize());
+
+    draw->DrawString(this->x+width-font2->StringWidth(packageSize)+1,
+                     this->y+Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
+                     font1->height+
+                     Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder)+
+                     font2->ascent,
+                     packageSize);
+
+    draw->PopFont();
+
+    draw->PopForeground();
+  }
+};
+
+class PackageDataProvider : public PackageModel::DataProvider
+{
+private:
+  PackageObject *object;
+
+public:
+  PackageDataProvider()
+  : object(new PackageObject())
+  {
+    object->SetFlex(true,true);
+    object->SetMinWidth(Lum::Base::Size::stdCharWidth,15);
+  }
+
+  virtual ~PackageDataProvider()
+  {
+    delete object;
+  }
+
+  Lum::Object* GetObject(const PackageModel::Iterator& iter, size_t column) const
   {
     switch (column) {
     case 1:
-      return Lum::TableView::left;
-      break;
-    case 2:
-      return Lum::TableView::right;
-      break;
+      object->SetPackage(*iter);
+      return object;
     default:
-      assert(false);
-      break;
+      return NULL;
     }
   }
 };
 
-class PackageTable : public Lum::Model::StdTable<PackageEntry>
+class PackageTable : public PackageModel
 {
 public:
-  class PackageDataProvider : public Lum::Model::StdTable<PackageEntry>::DataProvider
-  {
-    std::wstring GetString(const PackageTable::Iterator& iter, size_t column) const
-    {
-      return iter->GetString(column);
-    }
-
-    Lum::Object* GetObject(const PackageTable::Iterator& iter, size_t column) const
-    {
-      return iter->GetObject(column);
-    }
-  };
-
-  class PackageComparator : public Lum::Model::StdTable<PackageEntry>::Comparator
+  class PackageComparator : public PackageModel::Comparator
   {
   public:
-    bool operator()(const PackageEntry& a, const PackageEntry& b, size_t column, bool down) const
+    bool operator()(const PackageModel::ElementType& a,
+                    const PackageModel::ElementType& b,
+                    size_t column, bool down) const
     {
-      return a.GetPackage()->GetName()<b.GetPackage()->GetName();
+      return a->GetName()<b->GetName();
     }
   };
 
 public:
   PackageTable()
-  : Lum::Model::StdTable<PackageEntry>(new PackageDataProvider(),
-                                       new PackageComparator())
+  : PackageModel(new PackageDataProvider(),
+                 new PackageComparator())
   {
     // no code
   }
@@ -149,23 +240,19 @@ public:
 
   virtual void PreInit()
   {
-    Lum::Panel            *vpanel,*hpanel;
+    Lum::Panel            *vpanel;
+    Lum::Label            *filter;
     Lum::Table            *table;
     Lum::Model::HeaderRef headerModel;
 
-    vpanel=Lum::VPanel::Create(true,true);
-
-    hpanel=Lum::HPanel::Create(true,false);
-    hpanel->Add(new Lum::Text(L"Filter:"));
-    hpanel->AddSpace();
-    hpanel->Add(Lum::String::Create(search,true,false)->ShowClearButton(true));
-
-    vpanel->Add(hpanel);
-    vpanel->AddSpace();
+    filter=Lum::Label::Create(true,false);
+    filter->AddLabel(L"Filter:",
+                     Lum::String::Create(search,true,false)
+                     ->ShowClearButton(true)
+                     ->SetLabel(L"Filter"));
 
     headerModel=new Lum::Model::HeaderImpl();
-    headerModel->AddColumn(L"Package",Lum::Base::Size::stdCharWidth,28,true);
-    headerModel->AddColumn(L"Version",Lum::Base::Size::stdCharWidth,12,false);
+    headerModel->AddColumn(L"Package",Lum::Base::Size::stdCharWidth,35,true);
 
     table=new Lum::Table();
     table->SetFlex(true,true);
@@ -175,12 +262,24 @@ public:
     table->SetHeaderModel(headerModel);
     table->SetSelection(selection);
     table->SetDoubleClickAction(showPackageAction);
-    table->SetShowHeader(true);
-    table->SetFormatProvider(new PackageFormatProvider());
+    table->SetShowHeader(false);
     table->GetTableView()->SetAutoFitColumns(true);
     table->GetTableView()->SetAutoHSize(true);
 
-    vpanel->Add(table);
+    vpanel=Lum::VPanel::Create(true,true);
+
+    switch (Lum::OS::display->GetTheme()->GetToolbarPosition()) {
+    case Lum::OS::Theme::toolbarBottom:
+      vpanel->Add(table);
+      vpanel->AddSpace();
+      vpanel->Add(filter);
+      break;
+    default:
+      vpanel->Add(filter);
+      vpanel->AddSpace();
+      vpanel->Add(table);
+      break;
+    }
 
     SetMain(vpanel);
 
@@ -243,7 +342,7 @@ public:
          iter!=packages.end();
          ++iter) {
       if (search.empty() || iter->GetName().find(search)!=std::wstring::npos) {
-        currentPackages->Append(PackageEntry(&(*iter)));
+        currentPackages->Append(&(*iter));
       }
     }
 
@@ -260,7 +359,7 @@ public:
       packageListTask.Start();
     }
     else if (model==showPackageAction && showPackageAction->IsFinished()) {
-      Package*       package=currentPackages->GetEntry(selection->GetLine()).GetPackage();
+      Package*       package=currentPackages->GetEntry(selection->GetLine());
       PackageDetails *dialog;
 
       dialog=new PackageDetails(*package);
